package org.basex.modules.email;

import static org.basex.modules.email.EMailOptions.*;

import org.apache.commons.mail.*;
import org.basex.query.*;
import org.basex.query.func.*;
import org.basex.query.value.item.*;
import org.basex.util.*;

/**
 * EMail package.
 *
 * @author BaseX Team 2005-12, BSD License
 * @author Christian Gruen
 */
public final class EMail extends QueryModule {
  /** Element: email. */
  private static final QNm Q_EMAIL = new QNm("email");

  /**
   * Sends an email. Example:
   * <pre>
   * &lt;email&gt;
   *   &lt;from&gt;john@server.net&lt;/from&gt;
   *   &lt;to&gt;jean@client1.org&lt;/to&gt;
   *   &lt;to&gt;joe@client2.org&lt;/to&gt;
   *   &lt;host&gt;smtp.gmail.com&lt;/host&gt;
   *   &lt;user&gt;thatsme&lt;/user&gt;
   *   &lt;password&gt;topsecret&lt;/password&gt;
   *   &lt;subject&gt;Vacation&lt;/subject&gt;
   *   &lt;message&gt;Hi all, I am leaving for vacation...&lt;/message&gt;
   *   &lt;!-- optional: &lt;port&gt;587&lt;/port&gt; --&gt;
   *   &lt;!-- optional: &lt;ssl&gt;true&lt;/ssl&gt; --&gt;
   *   &lt;!-- optional: &lt;tls&gt;true&lt;/tls&gt; --&gt;
   *   &lt;!-- optional: &lt;reply-to&gt;...&lt;/reply-to&gt; --&gt;
   *   &lt;!-- optional: &lt;attachment&gt;/path/to/file.jpg&lt;/attachment&gt; --&gt;
   * &lt;/email&gt;
   * </pre>
   * @param it item
   * @throws QueryException query exception
   */
  public void send(final Item it) throws QueryException {
    final Thread ct = Thread.currentThread();
    final ClassLoader cl = ct.getContextClassLoader();

    final EMailOptions email = new EMailOptions();
    new FuncOptions(Q_EMAIL, null).assign(it, email);
    try {
      final MultiPartEmail mpe = new MultiPartEmail();
      mpe.setCharset(Strings.UTF8);

      if(email.contains(SUBJECT)) mpe.setSubject(email.get(SUBJECT));
      if(email.contains(MESSAGE)) mpe.setMsg(email.get(MESSAGE));
      if(email.contains(HOST)) mpe.setHostName(email.get(HOST));
      if(email.contains(PORT)) mpe.setSmtpPort(email.get(PORT));
      if(email.contains(FROM)) mpe.setFrom(email.get(FROM));
      if(email.contains(SSL)) mpe.setSSL(email.get(SSL));
      if(email.contains(TLS)) mpe.setTLS(email.get(TLS));
      for(final String s : email.get(TO)) mpe.addTo(s.trim());
      for(final String s : email.get(REPLY_TO)) mpe.addReplyTo(s.trim());
      for(final String s : email.get(ATTACHMENT)) {
        final EmailAttachment ea = new EmailAttachment();
        ea.setPath(s.trim());
        mpe.attach(ea);
      }
      if(email.contains(USER) && email.contains(PASSWORD)) {
        mpe.setAuthentication(email.get(USER), email.get(PASSWORD));
      }

      ct.setContextClassLoader(getClass().getClassLoader());
      mpe.send();
    } catch(final EmailException ex) {
      throw new QueryException("EMail: " + Util.message(ex));
    } finally {
      ct.setContextClassLoader(cl);
    }
  }
}
